/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef DADQ_CUDA_H
#define DADQ_CUDA_H

#include "qubcuda.h"

typedef struct
{
  int Nstim;
  int Ns;
  int Nq;
  int Nb;
  float *gklt;
  float *dQdq;
  float *Aek;
  float *dAdq_kl;
  float *dAdq;
  float *local;
} dAdq_context;

extern "C" {

QUBCUDA_API dAdq_context* __cdecl dAdq_context_create(int Nstim, int Ns, int Nq);
QUBCUDA_API void __cdecl dAdq_context_destroy(dAdq_context *ctx);
QUBCUDA_API void __cdecl dAdq_context_set_dQdq(dAdq_context *ctx, double ***dQdq);
QUBCUDA_API void __cdecl dAdq_context_set_Aek(dAdq_context *ctx, double ****Aek);
QUBCUDA_API void __cdecl dAdq_context_set_gklt(dAdq_context *ctx, double **gklt);
QUBCUDA_API void __cdecl dAdq_context_get_dAdq(dAdq_context *ctx, double ***dAdq, int e);

void dAdq_zero_kernel_call(int Nstim, int Ns, int Nk, int Nq, float *gklt, float *dQdq, float *Aek, float *dAdq_kl, float *dAdq);
void dAdq_kl_kernel_call(int Ns, int Nk, int Nq, float *gklt, float *dQdq, float *Ak, float *dAdq_kl);
void dAdq_sum_kernel_call(int Ns, int Nk, int Nq, float *dAdq_kl, float *dAdq);

}


#endif
